--[[
	Author: 	derelky
	Date:		18.07.2019
	Version:	1.0
	
	History:
				v1.0 @ 18.07.2019 - initial implementation in FS 19
]]

MixerWagonconf = {};

function MixerWagonconf.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations) and SpecializationUtil.hasSpecialization(TurnOnVehicle, specializations);
end

function MixerWagonconf.registerFunctions(vehicleType)
end

function MixerWagonconf.registerOverwrittenFunctions(vehicleType)    
end

function MixerWagonconf.initSpecialization()
    g_configurationManager:addConfigurationType("mixerWagon", g_i18n:getText("configuration_mixerWagon"), "mixerWagon", nil, nil, nil, ConfigurationUtil.SELECTOR_MULTIOPTION)
end

function MixerWagonconf.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", MixerWagonconf)
end




function MixerWagonconf:onPostLoad(savegame)
    local mixerWagonConfigurationId = Utils.getNoNil(self.configurations["mixerWagon"], 1)
    local baseKey = string.format("vehicle.mixerWagon.mixerWagonConfigurations.mixerWagonConfiguration(%d).mixerWagonFillTypes", mixerWagonConfigurationId -1)
    ObjectChangeUtil.updateObjectChanges(self.xmlFile, "vehicle.mixerWagon.mixerWagonConfigurations.mixerWagonConfiguration", mixerWagonConfigurationId , self.components, self)
	local spec = self.spec_mixerWagon	
    spec.fillUnitIndex = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.mixerWagon#fillUnitIndex"), 1)
	self.addFillUnitFillLevel = Utils.overwrittenFunction(self.addFillUnitFillLevel, MixerWagonconf.addFillUnitFillLevel);
	

	-- -- remove grass_windrow from fillTypes, we do not want to support grass in mixer wagons becasue this would make hay "useless"
    -- local fillUnit = self:getFillUnitByIndex(spec.fillUnitIndex)
    -- if fillUnit ~= nil then
        -- fillUnit.needsSaving = false
        -- if fillUnit.supportedFillTypes[FillType.GRASS_WINDROW] then
            -- fillUnit.supportedFillTypes[FillType.GRASS_WINDROW] = nil
        -- end
    -- end
    -- -- disable sync of fillLevels for fillUnit - will be handled by mixerWagon
    -- fillUnit.synchronizeFillLevel = false
	
    spec.mixerWagonFillTypes = {}
    spec.fillTypeToMixerWagonFillType = {}
	spec.outputfillType = g_fillTypeManager:getFillTypeIndexByName("FORAGE")
	
	local sumRatio = 0
    local i=0
    while true do
        local baseName = string.format("%s.mixerWagonFillType(%d)", baseKey, i) --string.format("vehicle.mixerWagon.mixerWagonFillTypes.mixerWagonFillType(%d)", i)
        local fillTypeStroutput = getXMLString(self.xmlFile,string.format("%s.mixerWagonoutput.fillType#fillType", baseKey)) --string.format("vehicle.mixerWagon.mixerWagonFillTypes.mixerWagonFillType(%d)", i)
		spec.outputfillType = g_fillTypeManager:getFillTypeIndexByName(fillTypeStroutput)
        if not hasXMLProperty(self.xmlFile, baseName) then
            break
        end
        local entry = {}
        entry.fillTypes = {}
        local j=0
        while true do
            local fillTypeKey = baseName..string.format(".fillType(%d)", j)
            if not hasXMLProperty(self.xmlFile, fillTypeKey) then
                break
            end
            local fillTypeStr = getXMLString(self.xmlFile, fillTypeKey.."#fillType")
            if fillTypeStr ~= nil then
                local fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(fillTypeStr)
                -- we do not want to support grass in mixer wagons becasue this would make hay "useless"
                if fillTypeIndex ~= nil and fillTypeIndex ~= FillType.GRASS_WINDROW then
                    if spec.fillTypeToMixerWagonFillType[fillTypeIndex] == nil then
                        entry.fillTypes[fillTypeIndex] = true
                        spec.fillTypeToMixerWagonFillType[fillTypeIndex] = entry
                    else
                        g_logManager:xmlWarning(self.configFileName, "MixerWagonFillType '%s' in '%s' already used! Ignoring it!", fillTypeKey, fillTypeStr)
                    end
                else
                    g_logManager:xmlWarning(self.configFileName, "FillType '%s' not defined for mixerWagonFillType '%s'!", fillTypeStr, fillTypeKey)
                end
            end
            j = j+1
        end
        entry.name = Utils.getNoNil(getXMLString(self.xmlFile, baseName.."#name"), "unknown")
        entry.minPercentage = Utils.getNoNil(getXMLFloat(self.xmlFile, baseName.."#minPercentage"), 0)*0.01
        entry.maxPercentage = Utils.getNoNil(getXMLFloat(self.xmlFile, baseName.."#maxPercentage"), 100)*0.01
        local ratiobackup = entry.maxPercentage  - entry.minPercentage
        entry.ratio = Utils.getNoNil(getXMLFloat(self.xmlFile, baseName.."#ratio"), ratiobackup)
        entry.fillLevel = 0
        if next(entry.fillTypes) ~= nil then
            sumRatio = sumRatio + entry.ratio
            table.insert(spec.mixerWagonFillTypes, entry)
        end
        i = i+1
    end
    for i, entry in ipairs(spec.mixerWagonFillTypes) do
        entry.ratio = entry.ratio / sumRatio
    end
	
	
    --spec.dirtyFlag = self:getNextDirtyFlag()
    -- if savegame ~= nil then
        -- for i, entry in ipairs(spec.mixerWagonFillTypes) do
            -- local fillTypeKey = savegame.key..string.format(".mixerWagon.fillType(%d)#fillLevel", i-1)
            -- local fillLevel = Utils.getNoNil(getXMLFloat(savegame.xmlFile, fillTypeKey), 0)
            -- if fillLevel > 0 then
                -- self:addFillUnitFillLevel(self:getOwnerFarmId(), spec.fillUnitIndex, fillLevel, next(entry.fillTypes), ToolType.UNDEFINED, nil)
            -- end
        -- end
    -- end
	
end


function MixerWagonconf:addFillUnitFillLevel(superFunc, farmId, fillUnitIndex, fillLevelDelta, fillTypeIndex, toolType, fillPositionData)
	
    local spec = self.spec_mixerWagon
    if fillUnitIndex ~= spec.fillUnitIndex then
        return superFunc(self, farmId, fillUnitIndex, fillLevelDelta, fillTypeIndex, toolType, fillPositionData)
    end
    local oldFillLevel = self:getFillUnitFillLevel(fillUnitIndex)
    local mixerWagonFillType = spec.fillTypeToMixerWagonFillType[fillTypeIndex]
    -- allow to put forage back to mixer. Split it up into valid forage mixing ratios
    if fillTypeIndex == spec.outputfillType and fillLevelDelta > 0 then
        for _, entry in pairs(spec.mixerWagonFillTypes) do
            local delta = fillLevelDelta * entry.ratio
            self:addFillUnitFillLevel(farmId, fillUnitIndex, delta, next(entry.fillTypes), toolType, fillPositionData)
        end
        return fillLevelDelta
    end
    if mixerWagonFillType == nil then
        -- used for discharge
        if fillLevelDelta < 0 and oldFillLevel > 0 then
            -- remove values from all fill types such that the ratio doesn't change
            fillLevelDelta = math.max(fillLevelDelta, -oldFillLevel)
            local newFillLevel = 0
            for _, entry in pairs(spec.mixerWagonFillTypes) do
                local entryDelta = fillLevelDelta * (entry.fillLevel / oldFillLevel)
                entry.fillLevel = math.max(entry.fillLevel + entryDelta, 0)
                newFillLevel = newFillLevel + entry.fillLevel
            end
            if newFillLevel < 0.1 then
                for _, entry in pairs(spec.mixerWagonFillTypes) do
                    entry.fillLevel = 0
                end
                fillLevelDelta = -oldFillLevel
            end
            self:raiseDirtyFlags(spec.dirtyFlag)
            local ret = superFunc(self, farmId, fillUnitIndex, fillLevelDelta, fillTypeIndex, toolType, fillPositionData)
            return ret
        end
        return 0
    end
    local capacity = self:getFillUnitCapacity(fillUnitIndex)
    local free = capacity - oldFillLevel
    if fillLevelDelta > 0 then
        mixerWagonFillType.fillLevel = mixerWagonFillType.fillLevel + math.min(free, fillLevelDelta)
        spec.activeTimer = spec.activeTimerMax
    else
        mixerWagonFillType.fillLevel = math.max(0, mixerWagonFillType.fillLevel + fillLevelDelta)
    end
    local newFillLevel = 0
    for _, mixerWagonFillType in pairs(spec.mixerWagonFillTypes) do
        newFillLevel = newFillLevel + mixerWagonFillType.fillLevel
    end
    newFillLevel = MathUtil.clamp(newFillLevel, 0, self:getFillUnitCapacity(fillUnitIndex))
    local newFillType = FillType.UNKNOWN
    local isSingleFilled = false
    local isForageOk = false
    for _, mixerWagonFillType in pairs(spec.mixerWagonFillTypes) do
        if newFillLevel == mixerWagonFillType.fillLevel then
            isSingleFilled = true
            newFillType = next(mixerWagonFillType.fillTypes)
            break
        end
    end
    if not isSingleFilled then
        isForageOk = true
        for _, mixerWagonFillType in pairs(spec.mixerWagonFillTypes) do
            if mixerWagonFillType.fillLevel < mixerWagonFillType.minPercentage * newFillLevel - 0.01 or mixerWagonFillType.fillLevel > mixerWagonFillType.maxPercentage * newFillLevel + 0.01 then
                isForageOk = false
                break
            end
        end
    end
    if isForageOk then
        newFillType = spec.outputfillType
    elseif not isSingleFilled then
        newFillType = FillType.FORAGE_MIXING
    end
    self:raiseDirtyFlags(spec.dirtyFlag)
    self:setFillUnitFillType(fillUnitIndex, newFillType)
    return FillUnit.addFillUnitFillLevel(self, farmId, fillUnitIndex, newFillLevel-oldFillLevel, newFillType, toolType, fillPositionData)
end
